<?php
declare(strict_types=1);

namespace IntiTest\Wp\Provider\Mock;

use Inti\Provider\Query\QueryTrait;
use Inti\Provider\Type\TypeInterface;
use Inti\Wp\Provider\Post\AbstractPostProvider;

use function in_array;
use function preg_match;
use function preg_quote;

/**
 * Class PostProvider
 *
 * @package IntiTest\Wp\Provider\Mock
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-provider
 */
final class PostProvider extends AbstractPostProvider
{
    use QueryTrait;
    use MetadataProviderTrait;

    protected array $field_map = [
        'id' => 'ID',
    ];

    private array $items = [
        ['ID' => 1, 'post_title' => 'First post', 'post_name' => 'first-post'],
        ['ID' => 2, 'post_title' => 'Second post', 'post_name' => 'second-post'],
        ['ID' => 3, 'post_title' => 'Third post', 'post_name' => 'third-post'],
        ['ID' => 4, 'post_title' => 'Fourth post', 'post_name' => 'fourth-post'],
        ['ID' => 5, 'post_title' => 'Fifth post', 'post_name' => 'fifth-post'],
        ['ID' => 6, 'post_title' => 'Sixth post', 'post_name' => 'sixth-post'],
        ['ID' => 7, 'post_title' => 'Seventh post', 'post_name' => 'seventh-post'],
        ['ID' => 8, 'post_title' => 'Eighth post', 'post_name' => 'eighth-post'],
    ];

    public function __construct()
    {
        $this->initMetadata('post', 8, 2);
    }

    public function getProvidedType(): TypeInterface
    {
        return CustomPostType::instance();
    }

    protected function fetchResults(): array
    {
        $results = array_filter($this->items, function (array $item) {
            if (isset($this->_query['ids']) && !empty($this->_query['ids'])) {
                $itemId = $this->getId($item);
                return in_array($itemId, $this->_query['ids']);
            }

            if (isset($this->_query['search']) && !empty($this->_query['search'])) {
                $query = preg_quote($this->_query['search']);
                return preg_match("/{$query}/", $item['post_title']);
            }

            if (isset($this->_query['slugs']) && !empty($this->_query['slugs'])) {
                return in_array($item['post_name'], $this->_query['slugs']);
            }

            return true;
        });

        if (isset($this->_query['paginate'])) {
            [$page, $itemsPerPage] = $this->_query['paginate'];
            $results = array_chunk($results, $itemsPerPage);
            $results = $results[$page - 1];
        }

        return $results;
    }

    public function getCount(): int
    {
        return count($this->items);
    }
}
